/*
 * Grzegorz Smulko, F1ISI
 *
 * Biblioteka OTL uyta do komunikacji z baz PostgreSQL
 *
 * OTL (Open Template Library) jest bibliotek umoliwiajc dostp do baz danych Oracle i DB2, a take
 * do wielu innych baz danych za porednictwem ODBC. Baza danych PostgreSQL dostpna jest dla OTL jednynie 
 * za porednictwem interfejsu ODBC, zatem musi on by odpowiednio skonfigurowany.
 *
 * Wicej informacji mona uzyska:
 * - na stronie domowej projektu OTL - http://otl.sourceforge.net/
 * - na stronie domowej projektu PostgreSQL - http://www.postgresql.org/
 * 
 * Aby uruchomi poniszy przykad w rodowisku Windows, naley:
 * 1. pobra (http://otl.sourceforge.net/otlv4_h.zip) i umieci w odpowiedniej lokalizacji plik nagwkowy otlv4.h
 * 2. zainstalowa baz danych PostgreSQL (np. zgodnie z instrukcj na http://pginstaller.projects.postgresql.org/)
 *    razem ze sterownikami ODBC
 * 3. korzystajc np. z programu pgAdmin utworzy uytkownika i baz danych wykorzystywan przez ODBC
 * 4. stworzy i skonfigurowa rdo danych ODBC
 */

#include <iostream>
using namespace std;

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

// #define OTL_ODBC_UNIX // jeli UnixODBC jest uywany, naley usun komentarz z tej linii
#if !defined(_WINDOWS_) && !defined(_WIN64)
	#define OTL_ODBC
#else 
	#define OTL_ODBC_POSTGRESQL // wymagany z PG ODBC w systemie Windows
#endif

#include <otlv4.h> // doczenie pliku nagwkowego OTL 4.0

class otlPostgresTest
{
private:
	// obiekt poczenia z baz danych
	otl_connect db;
public:
	otlPostgresTest()
	{
		// inicjalizacja rodowiska ODBC
		 otl_connect::otl_initialize(); 
	}
	//logowanie do odbc, utworzenie testowej tabeli
	void otlConnect(const char* connection_string)
	{
		try{
			//logowanie do bazy danych korzystajc z podanego connection-stringa
			//moliwe jest utworzenie od razu obiektu otl_connect ( otl_connect db(connection_string); )
			// - wtedy wywoanie rlogon jest zbdne
			 db.rlogon(connection_string); 

			//usunicie testowej tabeli (jeli istnieje)
		  otl_cursor::direct_exec
		   (
			db,
			"drop table if exists test_tab"
			 //, otl_exception::disabled	// wycza moliwo rzucania wyjtkw OTL
											// byoby potrzebne, gdyby baza nie istniaa i nie byo klauzuli 'if exists'
		   ); 
		  //konieczny commit kadej instrukcji, chyba e wywoamy konstruktor 
		  //otl_connect(const char* connect_str,const int auto_commit=0) z auto_commit=1
		   db.commit();

		   // utworzenie testowej tabeli
		  otl_cursor::direct_exec(
			db,
			"create table test_tab(id int, text_value varchar(30))"
			);  
			db.commit();

		}
		 catch(otl_exception& p){	// przechwytuje wyjtek OTL
		  cerr<<p.msg<<endl;		// wypisuje komunikat bdu
		  cerr<<p.stm_text<<endl;	// wypisuje polecenie SQL, ktre spowodowao bd
		  cerr<<p.sqlstate<<endl;	// wypisuje komunikat SQLSTATE
		  cerr<<p.var_info<<endl;	// wypisuje zmienn, ktra spowodwaa bd
		 }
	}
	//tworzy i zwraca strumie z przygotowanym zapytaniem, ktre nastpnie naley uzupeni danymi
	otl_stream& getStream(const char* sql_statement, const int buffer_size = 1)
	{	//tworzy nowy strumie otl - po zakoczeniu uywania strumienia naley pamita o jego usuniciu (delete)
		//- w prawdziwym zastosowaniu wypadaoby uy np. sprytnego wskanika
		otl_stream* op = new otl_stream (
			buffer_size,	//rozmiar bufora zdefiniowany jako liczba wierszy, ktre maj by pobrane/wstawione/zaktualizowane 
							//podczas pojedynczej interakcji (round-trip) z baz danych	(ang. batch size, array size)
			sql_statement,	// zapytanie SQL
			db				// obiekt poczenia z baz danych
		);
		return *op;
	}
	//zwraca obiekt poczenia, aby moliwe byo wywoanie otl_stream ze skonfigurowanym otl_connect nie poprzez
	//metod klasy - burzc tym niestety zasad ukrywania danych
	otl_connect& getConnectObject()
	{
		return db;
	}

};


int main()
{
	otlPostgresTest opt;
	opt.otlConnect("postgres/postgres@PostgreSQL30W"); //connection-string w postaci: login/haslo@nazwaZrodlaDanychODBC
	
	//tworzenie zapyta do bazy danych oparte jest na koncepcji strumieni - najpierw tworzony jest strumie zawierajcy
	//szablon zapytania, a nastpnie wpisywane s do strumienia konkretne wartoci
	//ze wzgldu na fakt, e chcemy tworzy strumie w metodzie klasy testowej, tutaj musimy posugiwa si w odrobin
	//nienaturalny sposb wskanikem na strumie (niej przykad bez uycia getStream i wskanika na otl_stream)
	//tutaj: przykad wstawiania danych do bazy (insert)
	otl_stream* os = & opt.getStream("insert into test_tab values(:f1<int>,:f2<char[31]>)"); 
	char tmp[32];
	for(int i=1;i<=10;++i){
		sprintf(tmp,"Value%d",i);
		//wpisanie do strumienia konkretnych wartoci 
		//(ich typy musz by zgodne z podanymi w zapytaniu przy tworzeniu strumienia)
		(*os)<<i<<tmp;
	}
	//skasowanie strumienia utworzonego w funkcji getStream
	delete os;


	//tutaj: przykad aktualizacji danych (update) bez uycia getStream i wskanika na otl_stream
	//wada: konieczno jawnego okrelenia rozmiaru bufora i podania obiektu poczenia
	otl_stream os2(1, "UPDATE test_tab SET text_value=:f2<char[31]> WHERE id=:f1<int>", opt.getConnectObject());
		
	os2 << "Value changed" << 2;
	os2 << otl_null() << 3; 

	//tutaj: przykad pobierania danych (select)
	otl_stream* is = & opt.getStream("SELECT * FROM test_tab WHERE id>=:f1<int> and id<=:f2<int>", 20);//20=rozmiar bufora
		
	 (*is) << 2 << 6; // Wpisywanie danych wejciowych do strumienia

	 int id;
	 char text_value[31];

	while(!(*is).eof()){
		(*is) >>id >> text_value;
		cout<<"id="<<id<<", text_value=";
		if((*is).is_null())
			cout<<"NULL";
		else
			cout<<text_value;
		cout<<endl;
	}
	
	//skasowanie strumienia utworzonego w funkcji getStream
	delete is;

 //system("pause");
 return 0;
}